#!/bin/bash

DIRECTORY=/data/sensor-data
LOG_FILE=$DIRECTORY/sd_install.log
HOST_DATA_DIR_CONFIG=$DIRECTORY/config
HOST_DATA_DIR_DB=$DIRECTORY/db
HOST_DATA_DIR_LOGS=$DIRECTORY/logs
CONTAINER_NAME=sensor-data
APP_VERSION=max1.0-1.0.1
MIRROR=vofo
APP_IMAGE=$MIRROR/$CONTAINER_NAME:$APP_VERSION

PORT=6002

preCheck() {

    if [ ! -d $DIRECTORY ]; then
        # 如果目录不存在，创建目录
        mkdir -p $DIRECTORY
        echo "`date` mkdir $DIRECTORY success." >> $LOG_FILE
    else
        echo "`date` $DIRECTORY is exists." >> $LOG_FILE
    fi
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi

    if ! command -v curl > /dev/null; then
        echo "`date` curl command not found. Please install curl." | tee -a $LOG_FILE
        exit 1
    fi

    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    if [ ! -d $HOST_DATA_DIR_CONFIG ]; then
        # 如果目录不存在，创建目录
        mkdir -p $HOST_DATA_DIR_CONFIG
        echo "`date` mkdir $HOST_DATA_DIR_CONFIG success." | tee -a $LOG_FILE
    fi
    cp -f ./default.json /data/sensor-data/config/default.json
    return 0
}
    

install() {
    
    echo "`date` start install $CONTAINER_NAME on port $PORT" | tee -a $LOG_FILE
    # Create docker-compose.yml
    echo "version: '3'" > docker-compose.yml
    echo "services:" >> docker-compose.yml
    echo "  $CONTAINER_NAME:" >> docker-compose.yml
    echo "    image: $APP_IMAGE" >> docker-compose.yml
    echo "    container_name: $CONTAINER_NAME" >> docker-compose.yml
    echo "    restart: unless-stopped" >> docker-compose.yml
    echo "    network_mode: host" >> docker-compose.yml
    echo "    volumes:" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_CONFIG:/opt/sensor-data/config" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_DB:/opt/sensor-data/.config/foggie/db" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR_LOGS:/opt/sensor-data/.config/foggie/logs" >> docker-compose.yml
    
    # Start Sensor-data
    docker-compose up -d
    upResult=$?
    if [ $upResult -ne 0 ]; then
        echo "`date` install $CONTAINER_NAME failed retry 1" | tee -a $LOG_FILE
        sleep 1
        docker-compose up -d
        upRetryOneResult=$?
        if [ $upRetryOneResult -ne 0 ]; then
            echo "`date` install $CONTAINER_NAME failed retry 2" | tee -a $LOG_FILE
            sleep 1
            docker-compose up -d
            upRetryTwoResult=$?
            if [ $upRetryTwoResult -ne 0 ]; then
                echo "`date` install $CONTAINER_NAME failed with exit code $upRetryTwoResult" | tee -a $LOG_FILE
                return 1
            fi
        fi
    fi
    echo "`date` wait 10 seconds for $CONTAINER_NAME to start" | tee -a $LOG_FILE
    sleep 10
    
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME installation" | tee -a $LOG_FILE
    # Check Sensor-data status
    if ! docker ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` Sensor-data is not running." | tee -a $LOG_FILE
        return 1
    fi
    return 0
}

# step 1: pre-Check
preCheck
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Install Sensor-data
    install
    installResult=$?
    if [ $installResult -eq 0 ]
    then
        echo "`date` Install Sensor-data successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` Sensor-data installation process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 3
        fi
    else
        echo "`date` Install Sensor-data failed." | tee -a $LOG_FILE
        exit 2
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 1
fi