#!/bin/bash

DIRECTORY=/data/sensor-data
LOG_FILE=$DIRECTORY/sd_uninstall.log
HOST_DATA_DIR_CONFIG=$DIRECTORY/config
HOST_DATA_DIR_DB=$DIRECTORY/db
HOST_DATA_DIR_LOGS=$DIRECTORY/logs
CONTAINER_NAME=sensor-data
APP_VERSION=max1.0-1.0.1

preCheck() {
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi
    
    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    if ! command -v docker > /dev/null; then
        echo "`date` docker command not found. Please install docker." | tee -a $LOG_FILE
        exit 1
    fi
    return 0
}

uninstall() {
    # stop and rm docker
    if docker ps -a | grep $CONTAINER_NAME | grep Up > /dev/null; then
        contain_id=`docker ps | grep $CONTAINER_NAME | awk '{print $1}'`
        docker stop $contain_id
        stopResult=$?
        if [ $stopResult -ne 0 ]; then
            echo "`date` stop $contain_id failed with exit code $stopResult" | tee -a $LOG_FILE
            return 1
        fi
        echo "`date` wait 10 seconds for $CONTAINER_NAME to stop" | tee -a $LOG_FILE
        sleep 10
        docker rm $contain_id
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $contain_id failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi

    if docker ps -a | grep $CONTAINER_NAME | grep Exit > /dev/null; then
        contain_id=`docker ps -a | grep $CONTAINER_NAME | awk '{print $1}'`
        docker rm $contain_id
        rmResult=$?
        if [ $rmResult -ne 0 ]; then
            echo "`date` rm $contain_id failed with exit code $rmResult" | tee -a $LOG_FILE
            return 1
        fi
    fi
    
    # Clear docker images
    image_ids=$(docker images | grep $CONTAINER_NAME | awk '{print $3}')
    for id in $image_ids; do
        echo "`date` Deleting image with ID $id" | tee -a $LOG_FILE
        docker rmi $id
    done
    
    # Clear data
    rm -rf $HOST_DATA_DIR_DB
    rm -rf $HOST_DATA_DIR_LOGS
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME uninstallation" | tee -a $LOG_FILE
    
    # Check sensor-data status
    if docker-compose ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` sensor-data is running." | tee -a $LOG_FILE
        return 1
    fi

    # Clear docker images
    if docker images | grep $CONTAINER_NAME | grep $APP_VERSION > /dev/null; then
        echo "`date` sensor-data image is not removed." | tee -a $LOG_FILE
        return 1
    fi

    if [ -d "$HOST_DATA_DIR_DB" ]; then
        echo "`date` sensor-data db data is not removed." | tee -a $LOG_FILE
        return 1
    fi

    if [ -d "$HOST_DATA_DIR_LOGS" ]; then
        echo "`date` sensor-data log data is not removed." | tee -a $LOG_FILE
        return 1
    fi
    
    return 0
}

# step 1: pre-Check
preCheck
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Uninstall sensor-data
    uninstall
    uninstallResult=$?
    if [ $uninstallResult -eq 0 ]
    then
        echo "`date` Uninstall sensor-data successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` sensor-data uninstallation process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 9
        fi
    else
        echo "`date` Uninstall sensor-data failed." | tee -a $LOG_FILE
        exit 8
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 7
fi